;************************************************************************
;	DualThermo.asm Ver. 1.1						*
;									*
;	Gennaio 2003, Avezzano (AQ) Italia				*
;	Autore: Franco Anzini - email: franco.anzini@virgilio.it	*
;									*
;	Termometro interno-esterno con memoria delle temperature 	*
;	minime settimanali						*
;									*
;	Microprocessore 16F84						*
;	Quarzo 4.194.304 MHz						*
;									*
;************************************************************************

;Calcolo interrupt TMR0:
;Freq. 4194304 / 4 = 1048576 / 128 =  8192 / 256 = 32 interrupt ogni secondo


		ERRORLEVEL	-302
		PROCESSOR	16F84
		RADIX		HEX

		INCLUDE "P16F84.INC"

	__CONFIG _CP_OFF & _XT_OSC & _PWRTE_ON  & _WDT_OFF


;********************** Definizione label porte

#DEFINE		Buzzer		PORTA,0
#DEFINE		Libero1		PORTA,1
#DEFINE 	RS_LCD		PORTA,2
#DEFINE 	En_LCD		PORTA,3
#DEFINE 	Libero2		PORTA,4

#DEFINE		CTRL		PORTB,0		;Pulsante CTRL
#DEFINE 	INC		PORTB,1		;Pulsante INC

#DEFINE		Term_EXT	PORTB,2		;NTC esterno
#DEFINE		Term_INT	PORTB,3		;NTC Interno

#DEFINE		LCD_DB4		PORTB,4
#DEFINE 	LCD_DB5		PORTB,5
#DEFINE		LCD_DB6		PORTB,6
#DEFINE		LCD_DB7		PORTB,7

;********************** Direttive assembler, dati eeprom

		ORG	0x2100				;set data eeprom origin

	de	' ','D','i','g','i','t','a','l'		;Indirizzo 00 - 07
	de	' ','T','h','e','r','m','o',' '		;Indirizzo 08 - 0F
	de	0x00,'L','u','n',0x00,'M','a','r'	;Indirizzo 10 - 17
	de	0x00,'M','e','r',0x00,'G','i','o'	;Indirizzo 18 - 1F
	de	0x00,'V','e','n',0x00,'S','a','b'	;Indirizzo 20 - 27
	de	0x00,'D','o','m',0x00,' ',' ',' '	;Indirizzo 28 - 2F
	de	' ',' ',' ',' ',' ',' ',' ',' '		;Indirizzo 30 - 37
	de	' ',' ',' ',' ',' ',' ',' ',' '		;Indirizzo 38 - 3F

;********************** Bit di controllo X Flag_Reg
Prog		equ	0
Misura		equ	1
Tipo		equ	2

;********************** Registri generici

                ORG     0x0C

Secondi		res	1		;Indirizzo ram 0CH
Minuti		res	1
Ore		res	1
Giorno		res	1

Real_Ore	res	1		;Indirizzo ram 10H
Real_Minuti	res	1
Real_Giorno	res	1

Time		res	1		;Indirizzo ram 13H
Cursore		res	1
Flag_Reg	res	1

Reg_Tmp		res	1		;Indirizzo ram 16H
Real_Val	res	1
Value		res	1
Chk_Zero	res	1

Tmp_Lcd_Reg	res	1		;Indirizzo ram 1AH
Dly_Count_1	res	1
Dly_Count_2	res	1
Delay_User	res	1
Delay_Misura	res	1

Valore_Tmp	res	1		;Indirizzo ram 1FH
Decine		res	1
Unita		res	1

Lun_Temp	res	1		;Indirizzo ram 22H
Lun_Ore		res	1
Lun_Min		res	1

Mar_Temp	res	1		;Indirizzo ram 25H
Mar_Ore		res	1
Mar_Min		res	1

Mer_Temp	res	1		;Indirizzo ram 28H
Mer_Ore		res	1
Mer_Min		res	1

Gio_Temp	res	1		;Indirizzo ram 2BH
Gio_Ore		res	1
Gio_Min		res	1

Ven_Temp	res	1		;Indirizzo ram 2EH
Ven_Ore		res	1
Ven_Min		res	1

Sab_Temp	res	1		;Indirizzo ram 31H
Sab_Ore		res	1
Sab_Min		res	1

Dom_Temp	res	1		;Indirizzo ram 34H
Dom_Ore		res	1
Dom_Min		res	1

W_Tmp		res	1		;Indirizzo ram 37H
Status_Tmp	res	1
FSR_Tmp		res	1

;********************** Reset Vector

		ORG	0x00			;Start

		goto	Initialize		;Goto Initialize routine       

		ORG	0x04
             
		goto	Interrupt		;Goto Interrupt routine

;********************** Stringhe di testo X LCD
Testi_LCD
		addwf	PCL,F
Tst_Regola	dt	"Regola orario",0
Tst_INT		dt	" Int: ",0
Tst_EXT		dt	" Ext: ",0
Tst_Min		dt	"Minima ",0
Tst_Misura	dt	"Misura ",0
Tst_Interna	dt	"interna",0
Tst_Esterna	dt	"esterna",0

;********************** Routine di gestione interrupt da TMR0
Interrupt
		movwf	W_Tmp			;Salva il contenuto di W
		swapf	STATUS,W
		movwf	Status_Tmp
		swapf	FSR,W
		movwf	FSR_Tmp

		decfsz	Time,F
		goto	Ripristino
	
		movlw	0x20			;Offset Time = 32
		movwf	Time

		decf	Delay_User,F		;Decrementa registro ogni secondo

		incf	Delay_Misura,F		;Incrementa registro ogni secondo

		movlw	Secondi
		movwf	FSR			;Puntatore su registro Secondi
New_Cifra
		incf	INDF,F			;Registro successivo
		movlw	HIGH Limite		;PCLATH points to last page of memory
		movwf	PCLATH			;where the table sits
		movlw	Secondi
		subwf	FSR,W
		call	Limite			;Imposta limite sec, min ecc.
		subwf	INDF,W
		btfss	STATUS,Z
		goto	Ripristino
		clrf	INDF
		incf	FSR,F
		goto	New_Cifra

Ripristino
		swapf	Status_Tmp,W 
		movwf	STATUS
		swapf	FSR_Tmp,W
		movwf	FSR
		swapf	W_Tmp,F
		swapf	W_Tmp,W

		bcf	INTCON,T0IF		;Azzera TOIF
		retfie				;Return da interrupt

;**********************	Main loop
Main
		movlw	0x3C
		movwf	Delay_User		;Imposta ritardo 60 secondi
		clrf	Delay_Misura
Main_Loop
		call	View_Dey		;Visualizza orario

		btfss	CTRL
		goto	Prg_Orario		;Regolazione orario

		btfss	INC
		goto	Chk_Tasto		;Controllo tasto INC

		movf	Ore,F
                btfsc   STATUS,Z  
		call	Chk_Clear		;Controllo se mezzanotte

		btfsc	Delay_Misura,0	  
		call	Misura_INT		;Misura temperatura interna

		btfsc	Delay_Misura,1	  
		call	Misura_EXT		;Misura temperatura esterna

		movf	Delay_User,F		;Ritardo 60 secondi 
                btfss   STATUS,Z  
		goto	Main_Loop
Thermo
		movlw	0xC0
		call	Send_Cmd

		clrw
		call	Read_EE_Car		;Scrive su LCD Digital Thermo

		movlw	0x01
		movwf	Delay_User		;Imposta ritardo 1 secondo
Thermo_Loop
		call	View_Dey		;Visualizza orario

		btfss	CTRL
		goto	Prg_Orario		;Regolazione orario

		btfss	INC
		goto	Chk_Tasto		;Controllo tasto INC

		movf	Delay_User,F		;Ritardo 1 secondo
                btfss   STATUS,Z  
		goto	Thermo_Loop

;**********************	Se temperatura <= 0 gradi beep ogni 60 secondi

		movf	Chk_Zero,W
		sublw	0x8F			;Zero gradi posizione 143 tabella
		btfss	STATUS,C
		call	Beep

		goto	Main

;**********************	Reset minime giorno alle ore 24.00
Chk_Clear
		movf	Minuti,F
                btfss   STATUS,Z		;Se mezzanotte azzera temperatura  
		return
		movlw	HIGH Temp_Giorno	;PCLATH points to last page of memory
		movwf	PCLATH			;where the table sits
		call	Temp_Giorno

		movwf	FSR			;Puntatore su W
		clrf	INDF			;Assegna 39.0 gradi
		return

;**********************	Controllo tasto INC
Chk_Tasto
		movlw	0x03
		movwf	Delay_User		;Imposta ritardo 3 secondi
		call	Beep

		btfsc	INC
		goto	View_Minime		;Se rilasciato View_Minime

		call	View_Dey		;Visualizza orario
 
		movf	Delay_User,F		;Ritardo 3 secondi 
                btfss   STATUS,Z  
		goto	$ - 0x05

		call	Beep_2

		movlw	0xC1
		call	Send_Cmd

		movlw	Tst_Misura - 0x06
		call	View_Testo		;Scrive su LCD 'Misura'

		btfsc	Flag_Reg,Tipo		;Se interna passa ad esterna
		goto	Est_Flag

;*********************** Flag misura Tipo 1 = interna

		bsf	Flag_Reg,Tipo
		movlw	Tst_Interna - 0x06
		goto	View_Tipo

;*********************** Flag misura Tipo 0 = esterna
Est_Flag
		bcf	Flag_Reg,Tipo
		movlw	Tst_Esterna - 0x06

View_Tipo
		call	View_Testo		;Scrive su LCD interna o esterna

		movlw	0x01
		movwf	Delay_User		;Imposta ritardo 1 secondi

		call	View_Dey		;Visualizza orario

		movf	Delay_User,F		;Ritardo 1 secondi 
                btfss   STATUS,Z  
		goto	$ - 0x03

		goto	Main

;*********************** Visualizza temperature minime
View_Minime
		call	Clear			;Clear Display RAM
                movf	Giorno,W 
		movwf	Real_Giorno		;Salva giorno attuale		
                bsf     Flag_Reg,Prog		;Disabilita visualizzazione secondi
Scan_Minime
		movlw	Real_Ore
		call	Save_Orario

		movlw	HIGH Temp_Giorno	;PCLATH points to last page of memory
		movwf	PCLATH			;where the table sits
		call	Temp_Giorno

		movwf	FSR			;Puntatore su W

		movf	INDF,W
		movwf	Value			;Carica temperatura minima
		incf	FSR,W			;Incrementa FSR su W
		call	Load_Orario		;Carica l'orario della minima

		call	View_Dey		;Visualizza orario

		movlw	Real_Ore
		call	Load_Orario		;Recupera orario

		movlw	0xC1
		call	Send_Cmd

		movlw	Tst_Min - 0x06
		call	View_Testo		;Scrive su LCD 'Minima'

		call	Disp_Min		;Visualizza temperatura	

		call	Rilascio_INC		;Attende il rilascio di INC

		movlw	0x05
		movwf	Delay_User		;Imposta ritardo 5 secondi 

Loop_Minime
		btfss	CTRL
		goto	Fine_Man		;Esce

		movf	Delay_User,F		;Ritardo 5 secondi 
                btfsc   STATUS,Z  
		goto	Fine_Aut		;Esce

                btfsc   INC
		goto	Loop_Minime

		decf	Giorno,F		;Giorno precedente

		call	Beep

                movlw   0xFF
                subwf   Giorno,W		;Se Giorno = FFH salta 
                btfss   STATUS,Z  
		goto	Scan_Minime

                movlw   0x06
                movwf	Giorno			;Reimposta Giorno 
		goto	Scan_Minime

Fine_Man
		call	Beep
Fine_Aut
                movf	Real_Giorno,W 
		movwf	Giorno		
                bcf     Flag_Reg,Prog		;Abilita visualizzazione secondi
		goto	Thermo

;**********************	Misura tempera esterna
Misura_EXT
;**********************	Set PORTB 2 e 3 Output

		bsf	STATUS, RP0		;Bank 1	
		movlw	0x03
		movwf	TRISB
		bcf	STATUS, RP0		;Bank 0

		bsf	Term_EXT		;Carica condensatore temp EXT

		movlw	0x0A
		call 	Mil_Secondi		;Delay 10 msec

;**********************	Set PORTB 2 e 3 Input

		bsf	STATUS, RP0     	;Bank 1	
		movlw	0x0F
		movwf	TRISB
		bcf	STATUS, RP0     	;Bank 0

;**********************	Inizio misura temperatura esterna

		movlw	0x01
		movwf	Reg_Tmp
		clrf	Value
External	
		btfss	Value,0
		goto	Loop_MisEXT		;Compensazione non linearita'
		btfss	Value,1
		goto	Loop_MisEXT
		incf	Reg_Tmp,F
Loop_MisEXT
		call	Comp_Term		;Ritardo per compensazione
		btfss	Term_EXT
		goto	OK_EXT
		incfsz	Value,F			;Tempo scarica cond
		goto	External	
OK_EXT
		movf	Value,W			;Passa il valore a Chk_Zero per beep
		movwf	Chk_Zero

		call	Minime_Giorno		;Controllo minime		

		btfss	Flag_Reg,Tipo		;Controllo se INT o EXT	
		call	View_Temp

		clrf	Delay_Misura

		return

;**********************	Controllo temperature mimime
Minime_Giorno
		movlw	HIGH Temp_Giorno	;PCLATH points to last page of memory
		movwf	PCLATH			;where the table sits
		call	Temp_Giorno

		movwf	FSR			;Puntatore su temperatura giorno

		movf	Value,W			;Carica il valore su W
		subwf	INDF,W
		btfsc	STATUS,C		;Controllo se > del precedente
		return

		movf	Value,W
		movwf	INDF			;Se > aggiorna minima del giorno
		incf	FSR,W			;Registro successivo
		call	Save_Orario		;Salva l'ora della minima

		return

;**********************	Misura temperatura Interna
Misura_INT
		btfss	Flag_Reg,Tipo		;Se esterna incrementa Delay_Misura
		goto	Non_INT

;**********************	Set PORTB 2 e 3 Output

		bsf	STATUS, RP0		;Bank 1	
		movlw	0x03
		movwf	TRISB
		bcf	STATUS, RP0		;Bank 0

		bsf	Term_INT		;Carica condensatore temp INT

		movlw	0x0A
		call 	Mil_Secondi		;Delay 10 msec

;**********************	Set PORTB 2 e 3 Input

		bsf	STATUS, RP0     	;Bank 1	
		movlw	0x0F
		movwf	TRISB
		bcf	STATUS, RP0     	;Bank 0

;**********************	Inizio misura temperatura interna

		movlw	0x01
		movwf	Reg_Tmp
		clrf	Value
Internal	
		btfss	Value,0
		goto	Loop_MisINT		;Compensazione non linearita'
		btfss	Value,1
		goto	Loop_MisINT
		incf	Reg_Tmp,F
Loop_MisINT
		call	Comp_Term		;Ritardo per compensazione
		btfss	Term_INT
		goto	OK_INT
		incfsz	Value,F			;Tempo scarica cond
		goto	Internal	
OK_INT

;**********************	Visualizzazione Temperatura

View_Temp
		movlw	0xC1
		call	Send_Cmd

		btfsc	Flag_Reg,Tipo	
		movlw	Tst_INT - 0x06

		btfss	Flag_Reg,Tipo	
		movlw	Tst_EXT - 0x06

		call	View_Testo		;Scrive su LCD interna o esterna
Disp_Min
		movlw	HIGH GetTemp		;PCLATH points to last page of memory
		movwf	PCLATH			;where the table sits
		movf	Value,W

		call	CheckFF
		call	GetTemp

		movwf	Real_Val		;Temperatura su Real_Val
		btfss	Real_Val,7		;Controllo se segno +
		movlw	'+'

		btfsc	Real_Val,7		;Controllo se segno -
		movlw	'-'

		call	Send_Char

		movf	Real_Val,W
		movwf	Reg_Tmp			;W su Reg_Tmp
		swapf	Reg_Tmp,W
		andlw	0x03			;Preleva il valore decimale
		call	Send_Num

		movf	Real_Val,W
		movwf	Reg_Tmp			;W su Reg_Tmp
		andlw	0x0F			;Preleva il valore unitario
		call	Send_Num

		movlw	'.'
		call	Send_Char

		btfss	Real_Val,6		;Controllo se frazione 0
		movlw	'0'

		btfsc	Real_Val,6		;Controllo se frazione 5
		movlw	'5'

		call	Send_Char

;ATTENZIONE IL SIMBOLO GRADI POTREBBE NON ESSERE CORRETTO PER IL VOSTRO DISPLAY
;QUINDI SARA' NECESSARIO CAMBIARE IL VALORE B2H.

		movlw	0xB2			;Simbolo gradi
		call	Send_Char

		movlw	'C'
		call	Send_Char

		movlw	' '
		call	Send_Char

		return

;**********************	No misura interna
Non_INT
		incf	Delay_Misura,F

		return

;**********************	Regolazione orario
Prg_Orario
		movlw	0x05
		movwf	Delay_User		;Imposta ritardo 5 secondi

		btfsc	CTRL
		goto	Main			;Se rilasciato esce

		call	View_Dey		;Visualizza orario
 
		movf	Delay_User,F		;Ritardo 5 secondi 
                btfss   STATUS,Z  
		goto	$ - 0x05

		movlw	0x80			;GIE = 1 T0IE = 0 T0IF = 0
		movwf	INTCON			;Sospende interrupt in regolazione

                bsf     Flag_Reg,Prog		;Disabilita visualizzazione secondi

		call	Beep_2

                clrf	Secondi			;Azzera secondi
		call	Clear			;Clear Display RAM

		movlw	0xC1			;Posiziona su linea 2 carattere 1
		call	Send_Cmd

		movlw	Tst_Regola - 0x06
		call	View_Testo		;Scrive su LCD Regola orario

		call	View_Dey		;Visualizza orario

		call	Cursor_ON		;Abilita il cursore lampeggiante

		movlw	0x82			;Posiziona su linea 1 carattere 2
		call	Send_Cmd

		call	Rilascio_CTRL		;Attende il rilascio di CTRL

;**********************	Regolazione giorno
Prg_Giorno
                btfss   CTRL
		goto	Regola_Ore		;CTRL prosegue con la regolazione

                btfsc   INC
		goto	Prg_Giorno

		incf	Giorno,F

                movlw   0x07			;Se giorno = 7
                subwf   Giorno,W 
                btfsc   STATUS,Z  
		clrf	Giorno			;Clear Giorno = Lun
		call	Cursor_OFF		;Disabilita il cursore lampeggiante

		movlw	0x82			;Posiziona su linea 1 carattere 2
		call	Send_Cmd
		movlw	HIGH Get_Giorno		;PCLATH points to last page of memory
		movwf	PCLATH			;where the table sits
		call	Get_Giorno		;Controllo giorno
		call	Read_EE_Car		;Scrive su LCD giorno
		call	Beep
		call	Rilascio_INC		;Attende il rilascio di INC
		movlw	0x82			;Si posiziona su giorno
		call	Send_Cmd
		call	Cursor_ON		;Abilita il cursore lampeggiante

		goto	Prg_Giorno

;**********************	Regolazione ore
Regola_Ore

		movlw	0x87			;Posiziona su linea 1 carattere 7
		call	Send_Cmd
		call	Beep

		call	Rilascio_CTRL		;Attende il rilascio di CTRL
Prog_Ore
                btfss   CTRL
		goto	Regola_Min		;CTRL prosegue con la regolazione

                btfsc   INC
		goto	Prog_Ore

		movlw	0x86
		movwf	Cursore
		call	Reg_Ore

		goto	Prog_Ore

;**********************	Regolazione minuti
Regola_Min

		movlw	0x8A			;Posiziona su linea 1 carattere 10
		call	Send_Cmd
		call	Beep

		call	Rilascio_CTRL		;Attende il rilascio di CTRL
Prog_Min
                btfss   CTRL			;CTRL esce dalla regolazione
		goto	Fine_PRG		;rilasciare sincronizzando 0 secondi
						;con un orologio campione
                btfsc   INC
		goto	Prog_Min

		movlw	0x86
		movwf	Cursore
		call	Reg_Min

		goto	Prog_Min
Fine_PRG
		call	Cursor_OFF		;Disabilita il cursore lampeggiante
		movlw	0xA0			;GIE = 1 T0IE = 1 T0IF = 0
		movwf	INTCON			;Riattiva interrupt fine regolazione

Fine_Chk
		call	Beep
		call	Rilascio_CTRL		;Attende il rilascio di CTRL
		call	Clear			;Clear Display RAM
                bcf     Flag_Reg,Prog		;Abilita visualizzazione secondi
		goto	Thermo


;**********************	Incremento ore
Reg_Ore
		call	Beep
Rep_Ore
		incf	Ore,F
                movlw   0x18			;Se ore = 24
                subwf   Ore,W
                btfsc   STATUS,Z  
		clrf	Ore			;Clear ore
		call	Cursor_OFF		;Disabilita il cursore lampeggiante
		movf	Cursore,W
		call	XY_View
		movlw	0xDC			;Attende 220 millisecondi
		call	Mil_Secondi              
                btfss   INC
		goto	Rep_Ore

		movf	Cursore,W
		addlw	0x01
		call	Send_Cmd

		call	Cursor_ON		;Abilita il cursore lampeggiante
		return

;**********************	Incremento minuti
Reg_Min
		call	Beep
Rep_Min
		incf	Minuti,F
                movlw   0x3C			;Se minuti = 60
                subwf   Minuti,W 
                btfsc   STATUS,Z  
		clrf	Minuti			;Clear minuti
		call	Cursor_OFF		;Disabilita il cursore lampeggiante
		movf	Cursore,W
		call	XY_View
		movlw	0xDC			;Attende 220 millisecondi
		call	Mil_Secondi              
                btfss   INC
		goto	Rep_Min

		movf	Cursore,W
		addlw	0x04
		call	Send_Cmd
		call	Cursor_ON		;Abilita il cursore lampeggiante
		return

;**********************	Carica l'orario dell'indirizzo contenuto in W
Load_Orario
		movwf	FSR			;Puntatore su W
		movf	INDF,W
		movwf	Ore			;Carica su Ore il valore
	
		incf	FSR,F			;Puntatore sul successivo indirizzo
		movf	INDF,W
		movwf	Minuti			;Carica su Minuti il valore

		return
;**********************	Salva l'orario all'indirizzo contenuto in W
Save_Orario
		movwf	FSR			;Puntatore su W
		movf	Ore,W
		movwf	INDF
	
		incf	FSR,F			;Puntatore sul successivo indirizzo
		movf	Minuti,W
		movwf	INDF

		return

;**********************	Routine invio testi al display da eeprom
Read_EE_Car
		movwf	EEADR
Repeat_Read
		bsf	STATUS,RP0
		bsf	EECON1,RD
		bcf	STATUS,RP0
		movf	EEDATA,W		;Lettura carattere
		iorlw	0			;Controlla se  finita la stringa
		btfsc	STATUS,Z
		return				;Se  finita esce
		call	Send_Char		;Invia il carattere al display
		incf	EEADR,F
		goto	Repeat_Read	

;**********************	Routine invio testi al display da ram
View_Testo
		movwf	FSR			;Puntatore su W
View_Loop
		movlw	HIGH Testi_LCD		;PCLATH points to last page of memory
		movwf	PCLATH			;where the table sits
		movf	FSR,W
		incf	FSR,F			;Incrementa per prossimo carattere
		call	Testi_LCD		;Legge il carattere dalla stringa
		iorlw	0			;Controlla se  finita la stringa
		btfsc	STATUS,Z
		return				;Se  finita esce
		call	Send_Char		;Invia il carattere al display
		goto	View_Loop

;**********************	Routine scrittura LCD ore, minuti, secondi, giorno
View_Dey
		movlw	0x82			;Posiziona su linea 1 carattere 2
		call	Send_Cmd
		movlw	HIGH Get_Giorno		;PCLATH points to last page of memory
		movwf	PCLATH			;where the table sits
		call	Get_Giorno		;Controllo giorno
		call	Read_EE_Car		;Scrive su LCD giorno

		movlw	0x86			;Posiziona su linea 1 carattere 6
XY_View
		call	Send_Cmd

		movf	Ore,W			;Carica su W ore
		movwf	Valore_Tmp
		call	Hex_Dec

		movlw	":"
		call	Send_Char

		movf	Minuti,W		;Carica su W minuti
		movwf	Valore_Tmp
		call	Hex_Dec

		btfsc	Flag_Reg,Prog		;Se = 1 non visualizza secondi
		return

		movlw	":"
		call	Send_Char

		movf	Secondi,W		;Carica su W secondi
		movwf	Valore_Tmp

;********************** Routine conversione hex - dec
Hex_Dec
		clrf	Decine
		clrf	Unita
Convert_Dec
		movf	Valore_Tmp,W
		sublw	0x09			;Controlla se Decine > 9
		btfsc	STATUS,C		;se > 9 salta	
		goto	Convert_Uni
		incf	Decine,F		;Incrementa Decine
		movlw	0x0A
		subwf	Valore_Tmp,F		;Valore_Tmp - 10
		goto	Convert_Dec	

Convert_Uni
		movf	Valore_Tmp,W
		movwf	Unita

		movf	Decine,W		;Scrive su LCD Decine
		call	Send_Num
		movf	Unita,W			;Scrive su LCD Unita'

;**********************	Routine invio dati al display
Send_Num
		addlw	0x30			;Per inviare le cifre add 48

Send_Char
		bsf	RS_LCD			;RS_LCD = 1 invio dati
		goto	Send_Byte

;**********************	Routine clear display
Clear
		movlw	0x01			;Clear Display RAM
		goto	Send_Cmd

;**********************	Routine cursore OFF
Cursor_OFF
		movlw	0x0C			;Cursor OFF
		goto	Send_Cmd

;**********************	Routine cursore ON
Cursor_ON
		movlw	0x0F			;Cursor ON
		
;**********************	Routine invio comandi al display
Send_Cmd
		bcf	RS_LCD			;RS_LCD = 0 invio comandi
		
;**********************	Routine invio byte al display
Send_Byte
		movwf	Tmp_Lcd_Reg		;Legge da W il comando

		clrf	PORTB

		btfsc	Tmp_Lcd_Reg,4
		bsf	LCD_DB4
		btfsc	Tmp_Lcd_Reg,5
		bsf	LCD_DB5
		btfsc	Tmp_Lcd_Reg,6
		bsf	LCD_DB6
		btfsc	Tmp_Lcd_Reg,7
		bsf	LCD_DB7

		call	Enable_LCD

		clrf	PORTB

		btfsc	Tmp_Lcd_Reg,0
		bsf	LCD_DB4
		btfsc	Tmp_Lcd_Reg,1
		bsf	LCD_DB5
		btfsc	Tmp_Lcd_Reg,2
		bsf	LCD_DB6
		btfsc	Tmp_Lcd_Reg,3
		bsf	LCD_DB7

;********************** Abilitazione display
Enable_LCD
		bsf	En_LCD
		movlw	0x01			;Attende 1 millisecondo
		call	Mil_Secondi              
		bcf	En_LCD
		movlw	0x01			;Attende 1 millisecondo

;**********************	Routine ritardo millisec in base al contenuto di W
Mil_Secondi
                movwf   Dly_Count_2
                clrf    Dly_Count_1

                ;1 ms (about) internal loop
ms_Del_Loop
                nop
		decfsz	Dly_Count_1,F
		goto	ms_Del_Loop
                nop

		decfsz	Dly_Count_2,F
		goto	ms_Del_Loop

		return

;**********************	Routine ritardo in base al contenuto di W
Comp_Term
		movf	Reg_Tmp,W		;Reg_Tmp -> W
		addlw	0xFF
		movwf	Dly_Count_1		;W -> Dly_Count_1
Comp_Loop
		nop
                decfsz  Dly_Count_1,F
		goto	Comp_Loop

		return

;********************** Routine Beep
Beep_2
		call	Beep
		movlw	0xC8			;Delay beep 200 millisecondi
		call	Mil_Secondi              
Beep
		bcf	Buzzer
		movlw	0x32			;Beep 50 millisecondi
		call	Mil_Secondi              
		bsf	Buzzer
		return

;**********************	Controllo rilascio tasto CTRL
Rilascio_CTRL  
		btfss	Flag_Reg,Prog		;Se Prog = 1 salta
		call	View_Dey		;Visualizza orario
		btfss	CTRL			;Controlla CTRL
		goto	Rilascio_CTRL
		movlw	0x14			;Attende 20 millisecondi
		call	Mil_Secondi              
		btfss	CTRL			;Controlla se ancora basso
		goto	Rilascio_CTRL
		return				;Se tornato alto esce

;**********************	Controllo rilascio tasto INC
Rilascio_INC  
		btfss	Flag_Reg,Prog		;Se Prog = 1 salta
		call	View_Dey		;Visualizza orario
		btfss	INC			;Controlla INC
		goto	Rilascio_INC
		movlw	0x14			;Attende 20 millisecondi
		call	Mil_Secondi              
		btfss	INC			;Controlla se ancora basso
		goto	Rilascio_INC
		return				;Se tornato alto esce

;********************** Inizializzazione

Initialize
		bsf	STATUS,RP0		;Bank 1

		clrf	TRISA			;PORTA = '00000'

		movlw	0x03
		movwf	TRISB			;PORTB = '00000011'

		movlw	0x86			;Pull-up PORTB disabilitato                                    
						;prescaler assegnato a TMR0 1:128
		movwf	OPTION_REG                

		bcf	STATUS,RP0		;Bank 0

		clrf	PORTA			;Clear PORTA
		clrf	PORTB			;Clear PORTB
		bsf	Buzzer

;**********************	Azzeramento registri

		movlw	0x0C		
		movwf	FSR			;Puntatore sul primo registro 0x0C
Clr_Next_Reg
		clrf	INDF			;Clear file register
		incf	FSR,F			;Puntatore sul successivo indirizzo
		btfss	FSR,7			;FSR = 80 fine registri
		goto	Clr_Next_Reg        

		movlw	0x20			;Offset Time = 32, 1 sec = 32 interrupt
		movwf	Time

		movlw	0xA0			;GIE = 1 T0IE = 1 T0IF = 0
		movwf	INTCON

;********************** Inizializzazione display
LCD_Init
		movlw	0x28
		call	Mil_Secondi		;Attende 40 millisecondi

		movlw	0x03			;Reset
		call	Send_Cmd

		movlw	0x04
		call	Mil_Secondi		;Attende 4 millisecondi

		call	Enable_LCD		;Abilitazione display

		movlw	0x02
		call	Send_Cmd 
             
		movlw	0x28			;4 bits, 2 lines, 5x7 Font
		call	Send_Cmd

		call	Cursor_OFF

		movlw	0x06			;Increment cursor
		call	Send_Cmd

		call	Clear			;Clear Display RAM

		movlw	0x04
		call	Mil_Secondi		;Attende 4 millisecondi

		goto	Thermo

;********************** Routine controllo limite FF
CheckFF
		movwf	Reg_Tmp
		incfsz	Reg_Tmp,F
		return
		retlw	0xFE

;****************************************************************************************
;				Inizio tabelle assegnazione valori			*
;****************************************************************************************


;********************** Tabella assegnazione indirizzo ram registro giorno X FSR

		ORG    0x02E9
Temp_Giorno
                movf	Giorno,W 
		addwf	PCL,F			;Giorno su W + PCL -> PCL

		retlw	Lun_Temp		;Indirizzo temperatura Lunedi
		retlw	Mar_Temp		;Indirizzo temperatura Martedi
		retlw	Mer_Temp		;Indirizzo temperatura Mercoledi
		retlw	Gio_Temp		;Indirizzo temperatura Giovedi
		retlw	Ven_Temp		;Indirizzo temperatura Venerdi
		retlw	Sab_Temp		;Indirizzo temperatura Sabato
		retlw	Dom_Temp		;Indirizzo temperatura Domenica

;********************** Limite sec = 59, min = 59, ore = 23, gio = 6

		ORG    0x02F2
Limite
		addwf	PCL,F			;W + PCL -> PCL

		retlw	0x3C			;Secondi max 59
		retlw	0x3C			;Minuti max 59
		retlw	0x18			;Ore max 23
		retlw	0x07			;Giorno max 7

;********************** Tabella assegnazione indirizzo eeprom X LCD

		ORG    0x02F7
Get_Giorno
                movf	Giorno,W 
		addwf	PCL,F			;Giorno su W + PCL -> PCL

		retlw	0x11			;Indirizzo eeprom = Lun
		retlw	0x15			;Indirizzo eeprom = Mar
		retlw	0x19			;Indirizzo eeprom = Mer
		retlw	0x1D			;Indirizzo eeprom = Gio
		retlw	0x21			;Indirizzo eeprom = Ven
		retlw	0x25			;Indirizzo eeprom = Sab
		retlw	0x29			;Indirizzo eeprom = Dom


;********************** Tabella assegnazione valore temperatura

;	Formato del byte di ritorno

;	Bit		7		Segno + / -

;	Bit		6		Frazione 0 / 5

;	Bit		6 - 5		Decimali

;	Bit		3 - 0		Unita'


		ORG    0x0300
GetTemp

		addwf	PCL,F			;Valore su W + PCL -> PCL

;		Valore di ritorno	Value	Temperatura

		retlw	0x39		;0	+39
		retlw	0x78		;1	+38,5
		retlw	0x38		;2	+38
		retlw	0x77		;3	+37,5
		retlw	0x37		;4	+37
		retlw	0x76		;5	+36,5
		retlw	0x36		;6	+36
		retlw	0x75		;7	+35,5
		retlw	0x35		;8	+35
		retlw	0x74		;9	+34,5
		retlw	0x34		;10	+34
		retlw	0x34		;11	+34
		retlw	0x73		;12	+33,5
		retlw	0x33		;13	+33
		retlw	0x33		;14	+33
		retlw	0x72		;15	+32,5
		retlw	0x32		;16	+32
		retlw	0x32		;17	+32
		retlw	0x71		;18	+31,5
		retlw	0x31		;19	+31
		retlw	0x31		;20	+31
		retlw	0x70		;21	+30,5
		retlw	0x30		;22	+30
		retlw	0x30		;23	+30
		retlw	0x69		;24	+29,5
		retlw	0x29		;25	+29
		retlw	0x29		;26	+29
		retlw	0x68		;27	+28,5
		retlw	0x28		;28	+28
		retlw	0x28		;29	+28
		retlw	0x67		;30	+27,5
		retlw	0x27		;31	+27
		retlw	0x27		;32	+27
		retlw	0x66		;33	+26,5
		retlw	0x26		;34	+26
		retlw	0x26		;35	+26
		retlw	0x65		;36	+25,5
		retlw	0x65		;37	+25,5
		retlw	0x25		;38	+25
		retlw	0x25		;39	+25
		retlw	0x64		;40	+24,5
		retlw	0x64		;41	+24,5
		retlw	0x24		;42	+24
		retlw	0x24		;43	+24
		retlw	0x63		;44	+23,5
		retlw	0x63		;45	+23,5
		retlw	0x23		;46	+23
		retlw	0x23		;47	+23
		retlw	0x62		;48	+22,5
		retlw	0x62		;49	+22,5
		retlw	0x22		;50	+22
		retlw	0x22		;51	+22
		retlw	0x61		;52	+21,5
		retlw	0x61		;53	+21,5
		retlw	0x21		;54	+21
		retlw	0x21		;55	+21
		retlw	0x60		;56	+20,5
		retlw	0x60		;57	+20,5
		retlw	0x20		;58	+20
		retlw	0x20		;59	+20
		retlw	0x59		;60	+19,5
		retlw	0x59		;61	+19,5
		retlw	0x19		;62	+19
		retlw	0x19		;63	+19
		retlw	0x58		;64	+18,5
		retlw	0x58		;65	+18,5
		retlw	0x18		;66	+18
		retlw	0x18		;67	+18
		retlw	0x57		;68	+17,5
		retlw	0x57		;69	+17,5
		retlw	0x17		;70	+17
		retlw	0x17		;71	+17
		retlw	0x56		;72	+16,5
		retlw	0x56		;73	+16,5
		retlw	0x16		;74	+16
		retlw	0x16		;75	+16
		retlw	0x55		;76	+15,5
		retlw	0x55		;77	+15,5
		retlw	0x15		;78	+15
		retlw	0x15		;79	+15
		retlw	0x54		;80	+14,5
		retlw	0x54		;81	+14,5
		retlw	0x14		;82	+14
		retlw	0x14		;83	+14
		retlw	0x53		;84	+13,5
		retlw	0x53		;85	+13,5
		retlw	0x13		;86	+13
		retlw	0x13		;87	+13
		retlw	0x52		;88	+12,5
		retlw	0x52		;89	+12,5
		retlw	0x12		;90	+12
		retlw	0x12		;91	+12
		retlw	0x51		;92	+11,5
		retlw	0x51		;93	+11,5
		retlw	0x11		;94	+11
		retlw	0x11		;95	+11
		retlw	0x50		;96	+10,5
		retlw	0x50		;97	+10,5
		retlw	0x10		;98	+10
		retlw	0x10		;99	+10
		retlw	0x49		;100	+9,5
		retlw	0x49		;101	+9,5
		retlw	0x09		;102	+9
		retlw	0x09		;103	+9
		retlw	0x48		;104	+8,5
		retlw	0x48		;105	+8,5
		retlw	0x08		;106	+8
		retlw	0x08		;107	+8
		retlw	0x47		;108	+7,5
		retlw	0x47		;109	+7,5
		retlw	0x07		;110	+7
		retlw	0x07		;111	+7
		retlw	0x46		;112	+6,5
		retlw	0x46		;113	+6,5
		retlw	0x06		;114	+6
		retlw	0x06		;115	+6
		retlw	0x45		;116	+5,5
		retlw	0x45		;117	+5,5
		retlw	0x05		;118	+5
		retlw	0x05		;119	+5
		retlw	0x05		;120	+5
		retlw	0x44		;121	+4,5
		retlw	0x44		;122	+4,5
		retlw	0x04		;123	+4
		retlw	0x04		;124	+4
		retlw	0x04		;125	+4
		retlw	0x43		;126	+3,5
		retlw	0x43		;127	+3,5
		retlw	0x03		;128	+3
		retlw	0x03		;129	+3
		retlw	0x03		;130	+3
		retlw	0x42		;131	+2,5
		retlw	0x42		;132	+2,5
		retlw	0x02		;133	+2
		retlw	0x02		;134	+2
		retlw	0x02		;135	+2
		retlw	0x41		;136	+1,5
		retlw	0x41		;137	+1,5
		retlw	0x01		;138	+1
		retlw	0x01		;139	+1
		retlw	0x01		;140	+1
		retlw	0x40		;141	+0,5
		retlw	0x40		;142	+0,5
		retlw	0x00		;143	+0	beep
		retlw	0x00		;144	+0
		retlw	0x80		;145	0
		retlw	0x80		;146	0
		retlw	0xC0		;147	-0,5
		retlw	0xC0		;148	-0,5
		retlw	0x81		;149	-1
		retlw	0x81		;150	-1
		retlw	0x81		;151	-1
		retlw	0xC1		;152	-1,5
		retlw	0xC1		;153	-1,5
		retlw	0x82		;154	-2
		retlw	0x82		;155	-2
		retlw	0x82		;156	-2
		retlw	0xC2		;157	-2,5
		retlw	0xC2		;158	-2,5
		retlw	0x83		;159	-3
		retlw	0x83		;160	-3
		retlw	0x83		;161	-3
		retlw	0xC3		;162	-3,5
		retlw	0xC3		;163	-3,5
		retlw	0x84		;164	-4
		retlw	0x84		;165	-4
		retlw	0x84		;166	-4
		retlw	0xC4		;167	-4,5
		retlw	0xC4		;168	-4,5
		retlw	0x85		;169	-5
		retlw	0x85		;170	-5
		retlw	0x85		;171	-5
		retlw	0xC5		;172	-5,5
		retlw	0xC5		;173	-5,5
		retlw	0x86		;174	-6
		retlw	0x86		;175	-6
		retlw	0x86		;176	-6
		retlw	0xC6		;177	-6,5
		retlw	0xC6		;178	-6,5
		retlw	0x87		;179	-7
		retlw	0x87		;180	-7
		retlw	0x87		;181	-7
		retlw	0xC7		;182	-7,5
		retlw	0xC7		;183	-7,5
		retlw	0x88		;184	-8
		retlw	0x88		;185	-8
		retlw	0x88		;186	-8
		retlw	0xC8		;187	-8,5
		retlw	0xC8		;188	-8,5
		retlw	0x89		;189	-9
		retlw	0x89		;190	-9
		retlw	0x89		;191	-9
		retlw	0xC9		;192	-9,5
		retlw	0xC9		;193	-9,5
		retlw	0x90		;194	-10
		retlw	0x90		;195	-10
		retlw	0x90		;196	-10
		retlw	0xD0		;197	-10,5
		retlw	0xD0		;198	-10,5
		retlw	0xD0		;199	-10,5
		retlw	0x91		;200	-11
		retlw	0x91		;201	-11
		retlw	0x91		;202	-11
		retlw	0xD1		;203	-11,5
		retlw	0xD1		;204	-11,5
		retlw	0xD1		;205	-11,5
		retlw	0x92		;206	-12
		retlw	0x92		;207	-12
		retlw	0x92		;208	-12
		retlw	0xd2		;209	-12,5
		retlw	0xd2		;210	-12,5
		retlw	0xd2		;211	-12,5
		retlw	0x93		;212	-13
		retlw	0x93		;213	-13
		retlw	0x93		;214	-13
		retlw	0xD3		;215	-13,5
		retlw	0xD3		;216	-13,5
		retlw	0xD3		;217	-13,5
		retlw	0x94		;218	-14
		retlw	0x94		;219	-14
		retlw	0x94		;220	-14
		retlw	0xD4		;221	-14,5
		retlw	0xD4		;222	-14,5
		retlw	0xD4		;223	-14,5
		retlw	0x95		;224	-15
		retlw	0x95		;225	-15
		retlw	0x95		;226	-15
		retlw	0x95		;227	-15
		retlw	0xD5		;228	-15,5
		retlw	0xD5		;229	-15,5
		retlw	0xD5		;230	-15,5
		retlw	0x96		;231	-16
		retlw	0x96		;232	-16
		retlw	0x96		;233	-16
		retlw	0x96		;234	-16
		retlw	0xD6		;235	-16,5
		retlw	0xD6		;236	-16,5
		retlw	0xD6		;237	-16,5
		retlw	0xD6		;238	-16,5
		retlw	0x97		;239	-17
		retlw	0x97		;240	-17
		retlw	0x97		;241	-17
		retlw	0x97		;242	-17
		retlw	0xD7		;243	-17,5
		retlw	0xD7		;244	-17,5
		retlw	0xD7		;245	-17,5
		retlw	0xD7		;246	-17,5
		retlw	0x98		;247	-18
		retlw	0x98		;248	-18
		retlw	0x98		;249	-18
		retlw	0x98		;250	-18
		retlw	0xD8		;251	-18,5
		retlw	0xD8		;252	-18,5
		retlw	0xD8		;253	-18,5
		retlw	0xD8		;254	-18,5


	end
